from typing import Any, Dict, Optional


def make_app_info(app_name: str,
                  app_version: Optional[str]) -> Dict[str, Any]:
    payload = dict(
        name=app_name,
        version=app_version,
    )
    return payload



from app import db
from avatar_utils.db.mixins.time_cast import TimeCastMixin


class CreatedAtMixin:

    created_at = db.Column(db.DateTime(timezone=True),
                           default=TimeCastMixin.utc_now)
    updated_at = db.Column(db.DateTime(timezone=True),
                           default=TimeCastMixin.utc_now,
                           onupdate=TimeCastMixin.utc_now)



from abc import ABC, abstractmethod
from functools import partial
from logging import getLogger

logger = getLogger(__name__)


class CheckerBase(ABC):

    @abstractmethod
    def __init__(self, name: str, **kwargs):
        self.name: str = name
        self._kwargs: dict = kwargs

    @abstractmethod
    def checker(self, *args, **kwargs) -> bool:
        raise NotImplementedError

    @property
    def call(self) -> partial:
        return partial(self.checker, **self._kwargs)



from avatar_utils.healthz.checker_base import CheckerBase
from avatar_utils.healthz.postgresql_checker import PostgresqlChecker
from avatar_utils.healthz.routes import make_healthz_blueprint

__all__ = (
    'make_healthz_blueprint',
    'CheckerBase',
    'PostgresqlChecker',
)



from logging import getLogger
from typing import Dict, Optional

from flask import current_app
from requests import Session

from avatar_utils.core.lazy_cached_attribute import LazyCachedAttribute
from avatar_utils.sso_helper import FlaskAuthHeader

logger = getLogger()


class ServiceAccessor:

    def _make_url(self, rest):
        backend_url = current_app.config["BACKEND_URL"]
        return f'{backend_url}{rest}'

    _user_match_url = LazyCachedAttribute(method=_make_url, name='_user_match_url', rest='/da/user/match')
    _services_url = LazyCachedAttribute(method=_make_url, name='_user_match_url', rest='/proxy/services')
    _service_list_url = LazyCachedAttribute(method=_make_url, name='_user_match_url', rest='/proxy/service_list')
    _tags_url = LazyCachedAttribute(method=_make_url, name='_user_match_url', rest='/tags/categories')
    _user_logs_url = LazyCachedAttribute(method=_make_url, name='_user_match_url', rest='/logs/user')

    def user_match(self, json: Dict = None, **kwargs):
        return self.call_arbitrary_http(url=self._user_match_url, json=json, **kwargs)

    def services(self, json: Dict = None, **kwargs):
        return self.call_arbitrary_http(url=self._services_url, json=json, **kwargs)

    def service_list(self, json: Dict = None, **kwargs):
        return self.call_arbitrary_http(url=self._service_list_url, json=json, **kwargs)

    def tags(self, json: Dict = None, **kwargs):
        return self.call_arbitrary_http(url=self._tags_url, json=json, **kwargs)

    def user_logs(self, json: Dict = None, **kwargs):
        return self.call_arbitrary_http(url=self._user_logs_url, json=json, **kwargs)

    def call_arbitrary_http(self, url: str, json: dict = None, **kwargs) -> Optional[Dict]:
        session: Session = FlaskAuthHeader.make_session()
        response = session.post(url=url, json=json, **kwargs)

        if response.status_code != 200:
            logger.warning(f'Something went wrong, status code: {response.status_code}. Response: {response.text}')

        return response.json()


service_accessor = ServiceAccessor()



import logging

from avatar_utils.tests.UnitTest import UnitTest

logger = logging.getLogger(__name__)


class BaseUnitTest(UnitTest):
    from app import create_app, db

    app = create_app('testing')
    client = app.test_client()
    db = db



import requests

from avatar_utils.tests import RemoteApp
from avatar_utils.tests.BaseUnitTest import BaseUnitTest


class WithLoggedUser(BaseUnitTest):
    # test user credentials
    username = None
    password = None

    # actual test user id in used database
    test_token = None
    id_test_user = None

    def setUp(self):
        super().setUp()

        backend_url = self.app.config['BACKEND_URL']

        # login user
        response = RemoteApp.convert_to_flask_response(
            requests.post(f'{backend_url}/login', json=dict(username=self.username, password=self.password)))
        self.rprint(response)

        token = response.json['result']['token']
        self.test_token = token

        # get user_id
        response = RemoteApp.convert_to_flask_response(
            requests.get(f'{backend_url}/user', headers={'Authorization': f'Bearer {token}'}, verify=False))
        self.rprint(response)

        self.id_test_user = response.json['result']['id']['value']
        print(f'id_test_user = {self.id_test_user}')

        # add a token to headers
        auth_header = {'Authorization': f'Bearer {self.test_token}'}
        self.headers = {**self.headers, **auth_header}



from logging import getLogger
from typing import Optional

import requests
from flask import current_app

logger = getLogger()


class PlannerAdapter:

    @property
    def _planner_schedule_url(self):
        return f'{current_app.config["PLANNER_URL"]}/schedule'

    @property
    def _planner_unschedule_url(self):
        return f'{current_app.config["PLANNER_URL"]}/unschedule'

    def schedule_route_call(self, **kwargs) -> Optional[requests.Response]:
        try:
            kwargs = {
                'service_name': kwargs.get('service_name') or current_app.config["SERVICE_NAME"],
                'route': kwargs['route'],
                'route_args': kwargs.get('route_args') or {},
                'method': kwargs.get('method') or 'POST',
                'trigger_type': kwargs['trigger_type'],
                'trigger_args': kwargs['trigger_args']
            }

            response = requests.post(self._planner_schedule_url, json=kwargs)
            return response
        except Exception as err:
            logger.warning('Route was not schedule due to %s', err)

    def unschedule_route_call(self, **kwargs) -> Optional[requests.Response]:
        try:
            kwargs = {
                'service_name': kwargs.get('service_name') or current_app.config["SERVICE_NAME"],
                'route': kwargs['route'],
                'method': kwargs.get('method') or 'POST'
            }

            response = requests.post(self._planner_unschedule_url, json=kwargs)
            return response
        except Exception as err:
            logger.warning('Route was not unschedule due to %s', err)



from datetime import datetime, timezone, timedelta
from enum import Enum
from typing import Optional

SAINT_PETERSBURG_TIME_ZONE = timezone(timedelta(hours=3))


class DefaultTimezones(Enum):
    input = timezone.utc
    output = SAINT_PETERSBURG_TIME_ZONE


class TimeCastMixin:

    def datetime_field_to_iso(self,
                              field_name: str,
                              time_zone: timezone = DefaultTimezones.output.value) -> Optional[str]:
        value: datetime = getattr(self, field_name)
        if value is None:
            return
        if not isinstance(value, datetime):
            raise KeyError
        return self.datetime_to_iso(value, time_zone)

    def iso_field_to_datetime(self,
                              field_name: str,
                              time_zone: timezone = DefaultTimezones.output.value) -> Optional[datetime]:
        value: str = getattr(self, field_name)
        if value is None:
            return
        if not isinstance(value, str):
            raise KeyError
        return self.iso_to_datetime(value, time_zone)

    @staticmethod
    def datetime_to_iso(value: datetime,
                        time_zone: timezone = DefaultTimezones.output.value) -> str:
        if value.tzinfo is None:
            value = value.replace(tzinfo=DefaultTimezones.input.value)
        value = value.astimezone(time_zone)
        return value.isoformat()

    @staticmethod
    def iso_to_datetime(value: str,
                        time_zone: timezone = DefaultTimezones.output.value) -> datetime:
        dt = datetime.fromisoformat(value)
        if dt.tzinfo is None:
            dt.replace(tzinfo=DefaultTimezones.input.value)
        dt = dt.astimezone(time_zone)
        return dt

    @staticmethod
    def utc_now():
        return datetime.utcnow().replace(tzinfo=timezone.utc)
